library dob_input_field;

import 'package:flutter/material.dart';

enum DateFormatType { DDMMYYYY, YYYYMMDD, MMDDYYYY }

class DOBInputField extends StatefulWidget {
  DOBInputField({
    Key? key,
    DateTime? initialDate,
    required DateTime firstDate,
    required DateTime lastDate,
    this.showCursor = true,
    this.cursorRadius,
    this.cursorWidth = 2.0,
    this.cursorColor = Colors.black,
    this.onDateSubmitted,
    this.showLabel = false,
    this.dateFormatType = DateFormatType.DDMMYYYY,
    this.inputDecoration = const InputDecoration(
      border: OutlineInputBorder(
        borderSide: BorderSide.none,
      ),
      filled: true,
      fillColor: Colors.white,
      counterText: '',
      hintText: 'MM/DD/YYYY',
    ),
    this.onDateSaved,
    this.autovalidateMode = AutovalidateMode.onUserInteraction,
    this.textAlign = TextAlign.left,
    this.selectableDayPredicate,
    this.errorFormatText,
    this.errorInvalidText,
    this.fieldHintText,
    this.style,
    this.fieldLabelText,
    this.autofocus = false,
  })  : assert(firstDate != null),
        assert(lastDate != null),
        assert(autofocus != null),
        initialDate =
            initialDate != null ? DateUtils.dateOnly(initialDate) : null,
        firstDate = DateUtils.dateOnly(firstDate),
        lastDate = DateUtils.dateOnly(lastDate),
        super(key: key) {
    assert(
      !this.lastDate.isBefore(this.firstDate),
      'lastDate ${this.lastDate} must be on or after firstDate ${this.firstDate}.',
    );
    assert(
      initialDate == null || !this.initialDate!.isBefore(this.firstDate),
      'initialDate ${this.initialDate} must be on or after firstDate ${this.firstDate}.',
    );
    assert(
      initialDate == null || !this.initialDate!.isAfter(this.lastDate),
      'initialDate ${this.initialDate} must be on or before lastDate ${this.lastDate}.',
    );
    assert(
      selectableDayPredicate == null ||
          initialDate == null ||
          selectableDayPredicate!(this.initialDate!),
      'Provided initialDate ${this.initialDate} must satisfy provided selectableDayPredicate.',
    );
    assert(
      selectableDayPredicate == null ||
          initialDate == null ||
          selectableDayPredicate!(this.initialDate!),
      'Provided initialDate ${this.initialDate} must satisfy provided selectableDayPredicate.',
    );
  }

  /// show label when true,default is false.
  final bool showLabel;
  final DateFormatType dateFormatType;

  //style for text in field
  final TextStyle? style;

  ///show cursor,default is true
  final bool showCursor;

  ///  for cursor radius
  final Radius? cursorRadius;

  ///for cursor width
  final double cursorWidth;
  final Color cursorColor;

  ///want to show validation text,by default it's enabled and show validation text
  final AutovalidateMode autovalidateMode;
  final InputDecoration inputDecoration;

  /// If provided, it will be used as the default value of the field.
  final DateTime? initialDate;
  final TextAlign textAlign;

  /// The earliest allowable [DateTime] that the user can input.
  final DateTime firstDate;

  /// The latest allowable [DateTime] that the user can input.
  final DateTime lastDate;

  /// An optional method to call when the user indicates they are done editing
  /// the text in the field. Will only be called if the input represents a valid
  /// [DateTime].
  final ValueChanged<DateTime>? onDateSubmitted;

  /// An optional method to call with the final date when the form is
  /// saved via [FormState.save]. Will only be called if the input represents
  /// a valid [DateTime].
  final ValueChanged<DateTime>? onDateSaved;

  /// Function to provide full control over which [DateTime] can be selected.
  final SelectableDayPredicate? selectableDayPredicate;

  /// The error text displayed if the entered date is not in the correct format.
  final String? errorFormatText;

  /// The error text displayed if the date is not valid.
  ///
  /// A date is not valid if it is earlier than [firstDate], later than
  /// [lastDate], or doesn't pass the [selectableDayPredicate].
  final String? errorInvalidText;

  /// The hint text displayed in the [TextField].
  ///
  /// If this is null, it will default to the date format string. For example,
  /// 'mm/dd/yyyy' for en_US.
  final String? fieldHintText;

  /// The label text displayed in the [TextField].
  ///
  /// If this is null, it will default to the words representing the date format
  /// string. For example, 'Month, Day, Year' for en_US.
  final String? fieldLabelText;

  /// {@macro flutter.widgets.editableText.autofocus}
  final bool autofocus;

  @override
  State<DOBInputField> createState() => _DOBInputFieldState();
}

class _DOBInputFieldState extends State<DOBInputField> {
  final TextEditingController _controller = TextEditingController();
  DateTime? _selectedDate;
  String? _inputText;
  bool _autoSelected = false;

  @override
  void initState() {
    super.initState();
    _selectedDate = widget.initialDate;
  }

  @override
  void dispose() {
    _controller.dispose();
    super.dispose();
  }

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
    _updateValueForSelectedDate();
  }

  @override
  void didUpdateWidget(DOBInputField oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (widget.initialDate != oldWidget.initialDate) {
      // Can't update the form field in the middle of a build, so do it next frame
      WidgetsBinding.instance.addPostFrameCallback((Duration timeStamp) {
        setState(() {
          _selectedDate = widget.initialDate;
          _updateValueForSelectedDate();
        });
      });
    }
  }

  void _updateValueForSelectedDate() {
    if (_selectedDate != null) {
      final MaterialLocalizations localizations =
          MaterialLocalizations.of(context);
      _inputText = localizations.formatCompactDate(_selectedDate!);
      TextEditingValue textEditingValue =
          _controller.value.copyWith(text: _inputText);
      // Select the new text if we are auto focused and haven't selected the text before.
      if (widget.autofocus && !_autoSelected) {
        textEditingValue = textEditingValue.copyWith(
            selection: TextSelection(
          baseOffset: 0,
          extentOffset: _inputText!.length,
        ));
        _autoSelected = true;
      }
      _controller.value = textEditingValue;
    } else {
      _inputText = '';
      _controller.value = _controller.value.copyWith(text: _inputText);
    }
  }

  DateTime? _parseDate(
    String? text,
  ) {
    return _dateParse(text, dateFormatType: widget.dateFormatType);
  }

  int _getDaysInMonth(int year, int month) {
    if (month == DateTime.february) {
      final bool isLeapYear =
          (year % 4 == 0) && (year % 100 != 0) || (year % 400 == 0);
      if (isLeapYear) return 29;
      return 28;
    }
    const List<int> daysInMonth = <int>[
      31,
      -1,
      31,
      30,
      31,
      30,
      31,
      31,
      30,
      31,
      30,
      31
    ];
    return daysInMonth[month - 1];
  }

  _dateParse(String? inputString,
      {DateFormatType dateFormatType = DateFormatType.DDMMYYYY}) {
    if (inputString == null) {
      return null;
    }
    final List<String> inputParts = inputString.split('/');
    if (inputParts.length != 3) {
      return null;
    }

    switch (dateFormatType) {
      case DateFormatType.DDMMYYYY:
        final int? year = int.tryParse(inputParts[2], radix: 10);
        if (year == null || year < 1) {
          return null;
        }

        final int? month = int.tryParse(inputParts[1], radix: 10);
        if (month == null || month < 1 || month > 12) {
          return null;
        }

        final int? day = int.tryParse(inputParts[0], radix: 10);
        if (day == null || day < 1 || day > _getDaysInMonth(year, month)) {
          return null;
        }
        return DateTime(year, month, day);
      case DateFormatType.YYYYMMDD:
        final int? year = int.tryParse(inputParts[0], radix: 10);
        if (year == null || year < 1) {
          return null;
        }

        final int? month = int.tryParse(inputParts[1], radix: 10);
        if (month == null || month < 1 || month > 12) {
          return null;
        }

        final int? day = int.tryParse(inputParts[2], radix: 10);
        if (day == null || day < 1 || day > _getDaysInMonth(year, month)) {
          return null;
        }
        return DateTime(year, month, day);
      case DateFormatType.MMDDYYYY:

        // Assumes US mm/dd/yyyy format

        final int? year = int.tryParse(inputParts[2], radix: 10);
        if (year == null || year < 1) {
          return null;
        }

        final int? month = int.tryParse(inputParts[0], radix: 10);
        if (month == null || month < 1 || month > 12) {
          return null;
        }

        final int? day = int.tryParse(inputParts[1], radix: 10);
        if (day == null || day < 1 || day > _getDaysInMonth(year, month)) {
          return null;
        }
        return DateTime(year, month, day);
    }
  }

  bool _isValidAcceptableDate(DateTime? date) {
    return date != null &&
        !date.isBefore(widget.firstDate) &&
        !date.isAfter(widget.lastDate) &&
        (widget.selectableDayPredicate == null ||
            widget.selectableDayPredicate!(date));
  }

  String? _validateDate(
    String? text,
  ) {
    final DateTime? date = _parseDate(
      text,
    );
    if (date == null) {
      return widget.errorFormatText ??
          MaterialLocalizations.of(context).invalidDateFormatLabel;
    } else if (!_isValidAcceptableDate(date)) {
      return widget.errorInvalidText ??
          MaterialLocalizations.of(context).dateOutOfRangeLabel;
    }
    return null;
  }

  void _updateDate(String? text, ValueChanged<DateTime>? callback) {
    final DateTime? date = _parseDate(text);
    if (_isValidAcceptableDate(date)) {
      _selectedDate = date;
      _inputText = text;
      callback?.call(_selectedDate!);
    }
  }

  void _handleSaved(String? text) {
    _updateDate(text, widget.onDateSaved);
  }

  void _handleSubmitted(String text) {
    _updateDate(text, widget.onDateSubmitted);
  }

  @override
  Widget build(BuildContext context) {
    return TextFormField(
      showCursor: widget.showCursor,
      cursorRadius: widget.cursorRadius,
      cursorWidth: widget.cursorWidth,
      cursorColor: widget.cursorColor,
      style: widget.style,
      textAlign: widget.textAlign,
      autovalidateMode: widget.autovalidateMode,
      decoration: widget.showLabel
          ? widget.inputDecoration
              .copyWith(labelText: widget.fieldLabelText, hintText: _hintText())
          : widget.inputDecoration.copyWith(hintText: _hintText()),
      validator: _validateDate,
      keyboardType: TextInputType.datetime,
      onSaved: _handleSaved,
      maxLength: 10,
      onFieldSubmitted: _handleSubmitted,
      autofocus: widget.autofocus,
      controller: _controller,
    );
  }

  String _hintText() {
    switch (widget.dateFormatType) {
      case DateFormatType.DDMMYYYY:
        return "DD/MM/YYYY";
      case DateFormatType.YYYYMMDD:
        return "YYYY/MM/DD";
      case DateFormatType.MMDDYYYY:
        return "MM/DD/YYYY";
    }
  }
}
